/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores
 * CA 94065 USA or visit www.oracle.com if you need additional information or
 * have any questions.
 */

package com.codename1.components;

import com.codename1.ui.tree.Tree;

/**
 * Simple class showing off the filesystem as a tree component
 *
 * @author Shai Almog
 */
public class FileTree extends Tree {
    
    /**
     * Default constructor
     */
    public FileTree() {
        super(new FileTreeModel(true));
        setUIID("FileTree");
    }

    /**
     * Constructor with a model
     */
    public FileTree(FileTreeModel model) {
        super(model);
        setUIID("FileTree");
    }
    
    protected String childToDisplayLabel(Object child) {
        String s = child.toString();
        if(s.equals("/")) {
            return "Root";
        }
        if(s.endsWith("/")) {
            s = s.substring(0, s.length() - 1);
        }
        int l = s.lastIndexOf('/');
        return s.substring(l + 1);
    }
}
